from pypanther.rules.auth0.auth0_cic_credential_stuffing import Auth0CICCredentialStuffing as Auth0CICCredentialStuffing
from pypanther.rules.auth0.auth0_custom_role_created import Auth0CustomRoleCreated as Auth0CustomRoleCreated
from pypanther.rules.auth0.auth0_integration_installed import Auth0IntegrationInstalled as Auth0IntegrationInstalled
from pypanther.rules.auth0.auth0_mfa_factor_setting_enabled import (
    Auth0MFAFactorSettingEnabled as Auth0MFAFactorSettingEnabled,
)
from pypanther.rules.auth0.auth0_mfa_policy_disabled import Auth0MFAPolicyDisabled as Auth0MFAPolicyDisabled
from pypanther.rules.auth0.auth0_mfa_policy_enabled import Auth0MFAPolicyEnabled as Auth0MFAPolicyEnabled
from pypanther.rules.auth0.auth0_mfa_risk_assessment_disabled import (
    Auth0MFARiskAssessmentDisabled as Auth0MFARiskAssessmentDisabled,
)
from pypanther.rules.auth0.auth0_mfa_risk_assessment_enabled import (
    Auth0MFARiskAssessmentEnabled as Auth0MFARiskAssessmentEnabled,
)
from pypanther.rules.auth0.auth0_post_login_action_flow import Auth0PostLoginActionFlow as Auth0PostLoginActionFlow
from pypanther.rules.auth0.auth0_user_invitation_created import Auth0UserInvitationCreated as Auth0UserInvitationCreated
from pypanther.rules.auth0.auth0_user_joined_tenant import Auth0UserJoinedTenant as Auth0UserJoinedTenant
