from pypanther.rules.box.box_access_granted import BoxAccessGranted as BoxAccessGranted
from pypanther.rules.box.box_anomalous_download import BoxShieldAnomalousDownload as BoxShieldAnomalousDownload
from pypanther.rules.box.box_event_triggered_externally import (
    BoxEventTriggeredExternally as BoxEventTriggeredExternally,
)
from pypanther.rules.box.box_item_shared_externally import BoxItemSharedExternally as BoxItemSharedExternally
from pypanther.rules.box.box_malicious_content import BoxMaliciousContent as BoxMaliciousContent
from pypanther.rules.box.box_new_login import BoxNewLogin as BoxNewLogin
from pypanther.rules.box.box_policy_violation import (
    BoxContentWorkflowPolicyViolation as BoxContentWorkflowPolicyViolation,
)
from pypanther.rules.box.box_suspicious_login_or_session import BoxShieldSuspiciousAlert as BoxShieldSuspiciousAlert
from pypanther.rules.box.box_untrusted_device import BoxUntrustedDevice as BoxUntrustedDevice
from pypanther.rules.box.box_user_downloads import BoxLargeNumberDownloads as BoxLargeNumberDownloads
from pypanther.rules.box.box_user_permission_updates import (
    BoxLargeNumberPermissionUpdates as BoxLargeNumberPermissionUpdates,
)
