from pypanther.rules.zscaler.zia.zia_account_access_removal import ZIAAccountAccessRemoved as ZIAAccountAccessRemoved
from pypanther.rules.zscaler.zia.zia_additional_cloud_roles import ZIAAdditionalCloudRoles as ZIAAdditionalCloudRoles
from pypanther.rules.zscaler.zia.zia_backup_deleted import ZIABackupDeleted as ZIABackupDeleted
from pypanther.rules.zscaler.zia.zia_create_cloud_account import ZIACloudAccountCreated as ZIACloudAccountCreated
from pypanther.rules.zscaler.zia.zia_golden_restore_point_dropped import (
    ZIAGoldenRestorePointDropped as ZIAGoldenRestorePointDropped,
)
from pypanther.rules.zscaler.zia.zia_insecure_password_settings import (
    ZIAInsecurePasswordSettings as ZIAInsecurePasswordSettings,
)
from pypanther.rules.zscaler.zia.zia_log_streaming_disabled import ZIALogStreamingDisabled as ZIALogStreamingDisabled
from pypanther.rules.zscaler.zia.zia_logs_downloaded import ZIALogsDownloaded as ZIALogsDownloaded
from pypanther.rules.zscaler.zia.zia_password_expiration import ZIAPasswordExpiration as ZIAPasswordExpiration
from pypanther.rules.zscaler.zia.zia_trust_modification import ZIATrustModification as ZIATrustModification
