query bulkUploadDetectionsStatus($input: ID!) {
    bulkUploadDetectionsStatus(jobId: $input) {
        message
        status
        results {
            deletedRuleIds
            modifiedRuleIds
            newRuleIds
            totalRuleIds
        }
    }
}