from pypanther.rules.gsuite_activityevent.google_workspace_admin_custom_role import (
    GoogleWorkspaceAdminCustomRole as GoogleWorkspaceAdminCustomRole,
)
from pypanther.rules.gsuite_activityevent.google_workspace_advanced_protection_program import (
    GoogleWorkspaceAdvancedProtectionProgram as GoogleWorkspaceAdvancedProtectionProgram,
)
from pypanther.rules.gsuite_activityevent.google_workspace_apps_marketplace_allowlist import (
    GoogleWorkspaceAppsMarketplaceAllowlist as GoogleWorkspaceAppsMarketplaceAllowlist,
)
from pypanther.rules.gsuite_activityevent.google_workspace_apps_marketplace_new_domain_application import (
    GoogleWorkspaceAppsMarketplaceNewDomainApplication as GoogleWorkspaceAppsMarketplaceNewDomainApplication,
)
from pypanther.rules.gsuite_activityevent.google_workspace_apps_new_mobile_app_installed import (
    GoogleWorkspaceAppsNewMobileAppInstalled as GoogleWorkspaceAppsNewMobileAppInstalled,
)
from pypanther.rules.gsuite_activityevent.gsuite_advanced_protection import (
    GSuiteAdvancedProtection as GSuiteAdvancedProtection,
)
from pypanther.rules.gsuite_activityevent.gsuite_calendar_made_public import (
    GSuiteCalendarMadePublic as GSuiteCalendarMadePublic,
)
from pypanther.rules.gsuite_activityevent.gsuite_doc_ownership_transfer import (
    GSuiteDocOwnershipTransfer as GSuiteDocOwnershipTransfer,
)
from pypanther.rules.gsuite_activityevent.gsuite_external_forwarding import (
    GSuiteExternalMailForwarding as GSuiteExternalMailForwarding,
)
from pypanther.rules.gsuite_activityevent.gsuite_google_access import GSuiteGoogleAccess as GSuiteGoogleAccess
from pypanther.rules.gsuite_activityevent.gsuite_gov_attack import (
    GSuiteGovernmentBackedAttack as GSuiteGovernmentBackedAttack,
)
from pypanther.rules.gsuite_activityevent.gsuite_group_banned_user import GSuiteGroupBannedUser as GSuiteGroupBannedUser
from pypanther.rules.gsuite_activityevent.gsuite_leaked_password import GSuiteLeakedPassword as GSuiteLeakedPassword
from pypanther.rules.gsuite_activityevent.gsuite_login_type import GSuiteLoginType as GSuiteLoginType
from pypanther.rules.gsuite_activityevent.gsuite_mobile_device_compromise import (
    GSuiteDeviceCompromise as GSuiteDeviceCompromise,
)
from pypanther.rules.gsuite_activityevent.gsuite_mobile_device_screen_unlock_fail import (
    GSuiteDeviceUnlockFailure as GSuiteDeviceUnlockFailure,
)
from pypanther.rules.gsuite_activityevent.gsuite_mobile_device_suspicious_activity import (
    GSuiteDeviceSuspiciousActivity as GSuiteDeviceSuspiciousActivity,
)
from pypanther.rules.gsuite_activityevent.gsuite_passthrough_rule import GSuiteRule as GSuiteRule
from pypanther.rules.gsuite_activityevent.gsuite_suspicious_logins import (
    GSuiteSuspiciousLogins as GSuiteSuspiciousLogins,
)
from pypanther.rules.gsuite_activityevent.gsuite_two_step_verification import (
    GSuiteTwoStepVerification as GSuiteTwoStepVerification,
)
from pypanther.rules.gsuite_activityevent.gsuite_user_suspended import GSuiteUserSuspended as GSuiteUserSuspended
from pypanther.rules.gsuite_activityevent.gsuite_workspace_calendar_external_sharing import (
    GSuiteWorkspaceCalendarExternalSharingSetting as GSuiteWorkspaceCalendarExternalSharingSetting,
)
from pypanther.rules.gsuite_activityevent.gsuite_workspace_data_export_created import (
    GSuiteWorkspaceDataExportCreated as GSuiteWorkspaceDataExportCreated,
)
from pypanther.rules.gsuite_activityevent.gsuite_workspace_gmail_default_routing_rule import (
    GSuiteWorkspaceGmailDefaultRoutingRuleModified as GSuiteWorkspaceGmailDefaultRoutingRuleModified,
)
from pypanther.rules.gsuite_activityevent.gsuite_workspace_gmail_enhanced_predelivery_scanning import (
    GSuiteWorkspaceGmailPredeliveryScanningDisabled as GSuiteWorkspaceGmailPredeliveryScanningDisabled,
)
from pypanther.rules.gsuite_activityevent.gsuite_workspace_gmail_security_sandbox_disabled import (
    GSuiteWorkspaceGmailSecuritySandboxDisabled as GSuiteWorkspaceGmailSecuritySandboxDisabled,
)
from pypanther.rules.gsuite_activityevent.gsuite_workspace_password_enforce_strong_disabled import (
    GSuiteWorkspacePasswordEnforceStrongDisabled as GSuiteWorkspacePasswordEnforceStrongDisabled,
)
from pypanther.rules.gsuite_activityevent.gsuite_workspace_password_reuse_enabled import (
    GSuiteWorkspacePasswordReuseEnabled as GSuiteWorkspacePasswordReuseEnabled,
)
from pypanther.rules.gsuite_activityevent.gsuite_workspace_trusted_domains_allowlist import (
    GSuiteWorkspaceTrustedDomainsAllowlist as GSuiteWorkspaceTrustedDomainsAllowlist,
)
