from pypanther.rules.indicator_creation.new_aws_account_logging import (
    StandardNewAWSAccountCreated as StandardNewAWSAccountCreated,
)
from pypanther.rules.indicator_creation.new_user_account_logging import (
    StandardNewUserAccountCreated as StandardNewUserAccountCreated,
)
