from pypanther.rules.github.github_action_failed import GitHubActionFailed as GitHubActionFailed
from pypanther.rules.github.github_advanced_security_change import (
    GitHubAdvancedSecurityChange as GitHubAdvancedSecurityChange,
)
from pypanther.rules.github.github_branch_policy_override import (
    GitHubBranchPolicyOverride as GitHubBranchPolicyOverride,
)
from pypanther.rules.github.github_branch_protection_disabled import (
    GitHubBranchProtectionDisabled as GitHubBranchProtectionDisabled,
)
from pypanther.rules.github.github_org_auth_modified import GitHubOrgAuthChange as GitHubOrgAuthChange
from pypanther.rules.github.github_org_ip_allowlist import GitHubOrgIpAllowlist as GitHubOrgIpAllowlist
from pypanther.rules.github.github_org_moderators_add import GitHubOrgModeratorsAdd as GitHubOrgModeratorsAdd
from pypanther.rules.github.github_org_modified import GitHubOrgModified as GitHubOrgModified
from pypanther.rules.github.github_organization_app_integration_installed import (
    GithubOrganizationAppIntegrationInstalled as GithubOrganizationAppIntegrationInstalled,
)
from pypanther.rules.github.github_public_repository_created import (
    GithubPublicRepositoryCreated as GithubPublicRepositoryCreated,
)
from pypanther.rules.github.github_repo_collaborator_change import (
    GithubRepoCollaboratorChange as GithubRepoCollaboratorChange,
)
from pypanther.rules.github.github_repo_created import GithubRepoCreated as GithubRepoCreated
from pypanther.rules.github.github_repo_initial_access import GitHubRepoInitialAccess as GitHubRepoInitialAccess
from pypanther.rules.github.github_repo_ruleset_modified import GitHubRepoRulesetModified as GitHubRepoRulesetModified
from pypanther.rules.github.github_repo_visibility_change import (
    GithubRepoVisibilityChange as GithubRepoVisibilityChange,
)
from pypanther.rules.github.github_repo_vulnerability_dismissed import (
    GithubRepoVulnerabilityDismissed as GithubRepoVulnerabilityDismissed,
)
from pypanther.rules.github.github_repository_transfer import GithubRepositoryTransfer as GithubRepositoryTransfer
from pypanther.rules.github.github_secret_scanning_alert_created import (
    GitHubSecretScanningAlertCreated as GitHubSecretScanningAlertCreated,
)
from pypanther.rules.github.github_team_modified import GitHubTeamModified as GitHubTeamModified
from pypanther.rules.github.github_user_access_key_created import (
    GitHubUserAccessKeyCreated as GitHubUserAccessKeyCreated,
)
from pypanther.rules.github.github_user_role_updated import GitHubUserRoleUpdated as GitHubUserRoleUpdated
from pypanther.rules.github.github_webhook_modified import GitHubWebhookModified as GitHubWebhookModified
