from pypanther.rules.asana.asana_service_account_created import AsanaServiceAccountCreated as AsanaServiceAccountCreated
from pypanther.rules.asana.asana_team_privacy_public import AsanaTeamPrivacyPublic as AsanaTeamPrivacyPublic
from pypanther.rules.asana.asana_workspace_default_session_duration_never import (
    AsanaWorkspaceDefaultSessionDurationNever as AsanaWorkspaceDefaultSessionDurationNever,
)
from pypanther.rules.asana.asana_workspace_email_domain_added import (
    AsanaWorkspaceEmailDomainAdded as AsanaWorkspaceEmailDomainAdded,
)
from pypanther.rules.asana.asana_workspace_form_link_auth_requirement_disabled import (
    AsanaWorkspaceFormLinkAuthRequirementDisabled as AsanaWorkspaceFormLinkAuthRequirementDisabled,
)
from pypanther.rules.asana.asana_workspace_guest_invite_permissions_anyone import (
    AsanaWorkspaceGuestInvitePermissionsAnyone as AsanaWorkspaceGuestInvitePermissionsAnyone,
)
from pypanther.rules.asana.asana_workspace_new_admin import AsanaWorkspaceNewAdmin as AsanaWorkspaceNewAdmin
from pypanther.rules.asana.asana_workspace_org_export import AsanaWorkspaceOrgExport as AsanaWorkspaceOrgExport
from pypanther.rules.asana.asana_workspace_password_requirements_simple import (
    AsanaWorkspacePasswordRequirementsSimple as AsanaWorkspacePasswordRequirementsSimple,
)
from pypanther.rules.asana.asana_workspace_require_app_approvals_disabled import (
    AsanaWorkspaceRequireAppApprovalsDisabled as AsanaWorkspaceRequireAppApprovalsDisabled,
)
from pypanther.rules.asana.asana_workspace_saml_optional import AsanaWorkspaceSAMLOptional as AsanaWorkspaceSAMLOptional
