from pypanther.rules.wiz.wiz_alert_passthrough import WizAlertPassthrough as WizAlertPassthrough
from pypanther.rules.wiz.wiz_cicd_scan_policy_updated_or_deleted import (
    WizCICDScanPolicyUpdatedOrDeleted as WizCICDScanPolicyUpdatedOrDeleted,
)
from pypanther.rules.wiz.wiz_connector_updated_or_deleted import (
    WizConnectorUpdatedOrDeleted as WizConnectorUpdatedOrDeleted,
)
from pypanther.rules.wiz.wiz_data_classifier_updated_or_deleted import (
    WizDataClassifierUpdatedOrDeleted as WizDataClassifierUpdatedOrDeleted,
)
from pypanther.rules.wiz.wiz_image_integrity_validator_updated_or_deleted import (
    WizImageIntegrityValidatorUpdatedOrDeleted as WizImageIntegrityValidatorUpdatedOrDeleted,
)
from pypanther.rules.wiz.wiz_integration_updated_or_deleted import (
    WizIntegrationUpdatedOrDeleted as WizIntegrationUpdatedOrDeleted,
)
from pypanther.rules.wiz.wiz_revoke_user_sessions import WizRevokeUserSessions as WizRevokeUserSessions
from pypanther.rules.wiz.wiz_rotate_service_account_secret import (
    WizRotateServiceAccountSecret as WizRotateServiceAccountSecret,
)
from pypanther.rules.wiz.wiz_rule_change import WizRuleChange as WizRuleChange
from pypanther.rules.wiz.wiz_saml_identity_provider_change import (
    WizSAMLIdentityProviderChange as WizSAMLIdentityProviderChange,
)
from pypanther.rules.wiz.wiz_service_account_change import WizServiceAccountChange as WizServiceAccountChange
from pypanther.rules.wiz.wiz_update_ip_restrictions import WizUpdateIPRestrictions as WizUpdateIPRestrictions
from pypanther.rules.wiz.wiz_update_login_settings import WizUpdateLoginSettings as WizUpdateLoginSettings
from pypanther.rules.wiz.wiz_update_scanner_settings import WizUpdateScannerSettings as WizUpdateScannerSettings
from pypanther.rules.wiz.wiz_update_support_contact_list import (
    WizUpdateSupportContactList as WizUpdateSupportContactList,
)
from pypanther.rules.wiz.wiz_user_created_or_deleted import WizUserCreatedOrDeleted as WizUserCreatedOrDeleted
from pypanther.rules.wiz.wiz_user_role_updated_or_deleted import (
    WizUserRoleUpdatedOrDeleted as WizUserRoleUpdatedOrDeleted,
)
