from pypanther.rules.standard.admin_assigned import StandardAdminRoleAssigned as StandardAdminRoleAssigned
from pypanther.rules.standard.brute_force_by_ip import StandardBruteForceByIP as StandardBruteForceByIP
from pypanther.rules.standard.brute_force_by_user import StandardBruteForceByUser as StandardBruteForceByUser
from pypanther.rules.standard.impossible_travel_login import (
    StandardImpossibleTravelLogin as StandardImpossibleTravelLogin,
)
from pypanther.rules.standard.malicious_sso_dns_lookup import (
    StandardMaliciousSSODNSLookup as StandardMaliciousSSODNSLookup,
)
from pypanther.rules.standard.mfa_disabled import StandardMFADisabled as StandardMFADisabled
from pypanther.rules.standard.sign_in_from_rogue_state import (
    StandardSignInFromRogueState as StandardSignInFromRogueState,
)
from pypanther.rules.standard.standard_dns_base64 import StandardDNSBase64 as StandardDNSBase64
